/*
 * Copyright 2012 Francesco Cappelli
 *
 * This file is part of Pagliaio.
 *
 * Pagliaio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pagliaio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Pagliaio.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "include/DeleteAction.h"

#include <lucene++/LuceneHeaders.h>
#include <lucene++/FileUtils.h>

using namespace std;

extern Lucene::IndexWriterPtr indexWriter;

namespace pagliaio {

DeleteAction::DeleteAction() {}

DeleteAction::~DeleteAction() {}

string DeleteAction::generateResponse(void *cls, struct MHD_Connection *connection,
								 	 const char *url,
								 	 const char *method, const char *version,
								 	 const char *upload_data,
								 	 size_t *upload_data_size, void **con_cls)
{
	if (strncmp("GET", method, 3) == 0) {
		map<string, string>* args = new map<string, string>();
		MHD_get_connection_values(connection, MHD_GET_ARGUMENT_KIND, iterate_get, (void*)args);

		if (args->count("id") == 0) {
			addErrorToResponse("Numero o tipo degli argomenti errato.");
		}
		else {
			openlog("pagliaio", LOG_ODELAY, LOG_DAEMON);

			Lucene::String id = Lucene::StringUtils::toUnicode((*args)["id"].c_str());
			Lucene::TermPtr term = Lucene::newLucene<Lucene::Term>(L"id", id);

			indexWriter->deleteDocuments(term);
			indexWriter->commit();

			syslog(LOG_INFO, "Deleted document with id %ls", id.c_str());

			xmlNodePtr indexedDocs = addChildToResponse(xmlNewNode(NULL, BAD_CAST "indexedDocs"));

			stringstream ss;
			int docCount = 0;
			for (int i=0; i < indexWriter->getSegmentCount(); i++) {
				docCount += indexWriter->getDocCount(i);
			}
			ss << docCount;

			xmlAddChild(indexedDocs, xmlNewText(BAD_CAST ss.str().c_str()));
		}
	}
	else  {
		addErrorToResponse("Metodo " + string(method) + " non supportato!");
	}

	return this->toString();
}

}
