/*
 * Copyright 2012 Francesco Cappelli
 *
 * This file is part of Pagliaio.
 *
 * Pagliaio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pagliaio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Pagliaio.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ACTION_H_
#define ACTION_H_

#include <string>
#include <map>
#include <utils.h>

extern "C" {
#include <sys/time.h>
#include <stdint.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdarg.h>
#include <microhttpd.h>
#include <syslog.h>
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xmlerror.h>
#include <stdarg.h>
}

namespace pagliaio {

class Action {
private:
	xmlDocPtr response;
	xmlNodePtr responseEl;

protected:
	xmlNodePtr getResponseRoot() { return this->responseEl; }
	xmlDocPtr createEmptyResponse();
	void addErrorToResponse(xmlDocPtr responseDoc, std::string error);
	void addErrorToResponse(std::string error);
	xmlNodePtr addChildToResponse(xmlNodePtr child);
	void addAttributeToResponse(std::string name, std::string value);
	std::string toString();
	char* convertToMultiByte(const wchar_t* text);

public:
	Action();
	virtual ~Action();


	virtual std::string generateResponse(void *cls, struct MHD_Connection *connection,
										 const char *url,
										 const char *method, const char *version,
										 const char *upload_data,
										 size_t *upload_data_size, void **con_cls) = 0;
};

}

#endif /* ACTION_H_ */
