/*
 * Copyright 2012 Francesco Cappelli
 *
 * This file is part of Pagliaio.
 *
 * Pagliaio is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Pagliaio is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Pagliaio.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef PAGLIAIOQUERYPARSER_H_
#define PAGLIAIOQUERYPARSER_H_

#include <lucene++/LuceneHeaders.h>
namespace pagliaio {

class PagliaioQueryParser: public Lucene::QueryParser {
public:
	/// Constructs a query parser.
	/// @param matchVersion Lucene version to match.
	/// @param field The default field for query terms.
	/// @param analyzer Used to find terms in the query text.
	PagliaioQueryParser(Lucene::LuceneVersion::Version matchVersion, const Lucene::String& field, Lucene::AnalyzerPtr analyzer);

	/// Constructor with user supplied QueryParserCharStream.
	PagliaioQueryParser(Lucene::QueryParserCharStreamPtr stream);

	/// Constructor with generated Token Manager.
	PagliaioQueryParser(Lucene::QueryParserTokenManagerPtr tokenMgr);

	virtual ~PagliaioQueryParser();

protected:
	Lucene::QueryPtr getFieldQuery(const Lucene::String& field, const Lucene::String& queryText);
	Lucene::QueryPtr getFieldQuery(const Lucene::String& field, const Lucene::String& queryText, int32_t slop);
	Lucene::QueryPtr getRangeQuery(const Lucene::String& field, const Lucene::String& part1, const Lucene::String& part2, bool inclusive);
	Lucene::QueryPtr getBooleanQuery(Lucene::Collection<Lucene::BooleanClausePtr> clauses);
	Lucene::QueryPtr getBooleanQuery(Lucene::Collection<Lucene::BooleanClausePtr> clauses, bool disableCoord);
	Lucene::QueryPtr getWildcardQuery(const Lucene::String& field, const Lucene::String& termStr);
	Lucene::QueryPtr getPrefixQuery(const Lucene::String& field, const Lucene::String& termStr);
	Lucene::QueryPtr getFuzzyQuery(const Lucene::String& field, const Lucene::String& termStr, double minSimilarity);
};

}
#endif /* PAGLIAIOQUERYPARSER_H_ */
